#ifndef __CHANIH__
#define __CHANIH__

/*Header file of Chani (Charge transport simulator for rectangular surfaces).
 * 	Created: Apr 25, 2012 by Burak Budanur (burakbudanur@gmail.com)
*/

#include <iostream>
//Root libs:
#include "TMath.h"
#include "TMatrixD.h"
#include "TVectorD.h"
#include <cmath>
#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
//Chani functions:
#include "functions/parameters.C"
#include "functions/computelmn.C"
#include "functions/addConnector.C"
#include "functions/addCharge.C"
#include "functions/transient1.C"
#include "functions/transient2.C"
#include "functions/getV.C"
#include "functions/getQtotal.C"
#include "functions/capacitance.C"
#include "functions/getQxy.C"
//Chani classes:
#include "classes/Readp.C"
#include "classes/Findij.C"
//Global constants
#include "globalConsts.h"

using std::cout;
using std::endl;

void parameters(double lx, int nx, double ly, int ny, int Nm, double sigmas);

void computelmn();

void addConnector(double x, double y, double R);

void addCharge(double x, double y, double q, int tStep);

void transient1(int tStepi, int nofSteps, double Deltat);

void transient2(int tStepi, int nofSteps, double Deltat);

double capacitance();

double getV(int i);

double getQtotal(int tStep);

double getQxy(double x, double y, int tStep);

#endif // __CHANIH__
