#include "../Chani.h"
#include "../globalConsts.h"
#include "../classes/Readp.h"

double capacitance(){

	double C = 0;

	Readp rp;
	double lx = rp.Getlx();
	double ly = rp.Getly();
	double ax = rp.Getax();
	double ay = rp.Getay();
	int nx = rp.Getnx();
	int ny = rp.Getny();
	int Nm = rp.GetNm();
	
	int N = nx*ny;
	double DeltaSn = ax*ay;
	
	std::cout << "Reading the l-matrix" << std::endl;

	TMatrixD l(1,N,1,N);
	int nsub = N / Nm;

	for (int i = 1; i <= Nm; i++){
		for (int j = 1; j<= Nm; j++){
			
			TString fnamel = TString::Format("%slmatrix%i.root",dir, i+(j-1)*Nm);
						
			TFile f(fnamel); // open the input file.

			char label[12];
			sprintf(label, "l-matrix%i", i+(j-1)*Nm);

			TMatrixD *lp; // pointer of the sub matrix
			f.GetObject(label,lp);

			for (int m = (j-1)*nsub+1; m <= j*nsub; m++){
				for (int n = (i-1)*nsub+1; n<= i*nsub; n++){

					int mm = m - (j-1)*nsub;
					int nm = n - (i-1)*nsub;

					l(m,n) = (*lp)(mm,nm);

				}
			}

			delete lp;
			f.Close();
		}
	}

	std::cout << "Reading of the l-matrix is completed" << std::endl;

	TMatrixD linv(1,N,1,N);

	linv = l;

	std::cout << "Calculating the inverse matrix" << std::endl;

	linv.Invert();

	std::cout << "Matrix inversions is completed" << std::endl;

	TVectorD V(1,N);
	TVectorD sigma(1,N);
	
	V += 1; // surface with unit potential
	
	sigma = linv * V; // surface charge density distribution
	
	C = sigma.Sum() * DeltaSn; // capacitance

	TH2D * sigmah;
	sigmah = new TH2D("sigma(x,y)", "Charge distribution corresponding to unit potential", nx, -lx/2, lx/2, ny, -ly/2, ly/2);
	
	for (int j = 1; j<=ny; j++){
		for (int i = 1; i<=nx; i++){
			sigmah->SetCellContent(i, j, sigma(i+nx*(j-1)));
		}
	}
	
	TString fnameh = TString::Format("%sh.root", dir);

	TFile f(fnameh, "UPDATE"); // open the output file for histograms

	sigmah->Write("sigmahVconst"); // Write the histogram after transient calculation

	f.Close();
	
	delete sigmah;
	
	return C;
	
}
