#include "../Chani.h"
#include "../classes/Readp.h"

void transient2(int tStepi, int nofSteps, double Deltat){
	
	Readp rp;
	double lx = rp.Getlx();
	double ly = rp.Getly();
	double ax = rp.Getax();
	double ay = rp.Getay();
	int nx = rp.Getnx();
	int ny = rp.Getny();
	int Nm = rp.GetNm();
	double sigmas = rp.Getsigmas();

	int N = nx*ny;
	double DeltaSn = ax*ay;
	
	//std::cout << "Reading of the l-matrix" << std::endl;

	// Read the l-matrix:
	TMatrixD l(1,N,1,N);
	int nsub = N / Nm;

	for (int i = 1; i <= Nm; i++){
		for (int j = 1; j<= Nm; j++){
			
			TString fnamel = TString::Format("%slmatrix%i.root",dir, i+(j-1)*Nm);

			TFile f(fnamel); // open the input file. 

			char label[10];
			sprintf(label, "l-matrix%i", i+(j-1)*Nm);
			
			TMatrixD *lp; // pointer of the sub matrix
			f.GetObject(label,lp);
			
			for (int m = (j-1)*nsub+1; m <= j*nsub; m++){
				for (int n = (i-1)*nsub+1; n<= i*nsub; n++){
				
					int mm = m - (j-1)*nsub;
					int nm = n - (i-1)*nsub;

					l(m,n) = (*lp)(mm,nm);
					//std::cout << "l(" << m << "," << n << ") = " << l(m,n) << " ";

				}
			}
			
			delete lp;
			f.Close();
		}
	}
	
	//std::cout << "Reading of the l-matrix is completed" << std::endl; 
	
	//Read the initial surface charge density histogram:
	
	TH2 *sigmaht; // Surface charge density histogram
	sigmaht = new TH2D("SigmahtStep(x,y)", "Surface charge density" , nx, -lx/2, lx/2, ny, -ly/2, ly/2) ;
	
	TH2 *connh; //The connector histogram:
	connh = new TH2D("Connectors", "Connector histogram" , nx, -lx/2, lx/2, ny, -ly/2, ly/2) ;
		
	char fnameh[100];
	sprintf(fnameh, "%sh.root", dir);		
	TFile f(fnameh, "UPDATE"); // open the output file for histograms
	
	TH2 *sigmahtd;
	TH2 *connhd;
	
	char objName[16];
	sprintf(objName, "sigmah%i", tStepi); 
	//f.GetListOfKeys()->Print();
	
	f.GetObject(objName, sigmahtd); // Read the initial surface charge density histogram
	f.GetObject("connh", connhd); // Read the connector histogram
	
		
	TVectorD sigmavt(1,N); // sigma vector
	TVectorD Vvt(1,N); // potential vector		
	
	//Fill the sigma vector:
	


	for (int j = 1; j<=ny; j++){ 
		for (int i = 1; i<=nx; i++){	
			sigmaht->SetCellContent(i, j, sigmahtd->GetCellContent(i,j));
		}
	}

	for (int j = 1; j<=ny; j++){ 
		for (int i = 1; i<=nx; i++){	
			connh->SetCellContent(i, j, connhd->GetCellContent(i,j));
		}
	}
	
	delete sigmahtd;
	delete connhd;	
	
	for (int j = 1; j<=ny; j++){
		for (int i = 1; i<=nx; i++){
			sigmavt(i+nx*(j-1)) = sigmaht->GetCellContent(i,j);
		}
	}
		
	double Kt; // Surface current density variable
	double It; // Discharge current variable
	double Rcon; // Connector resistance variable
	double sumt; // Charge sum variable


	for (int it = tStepi + 1; it <= tStepi + nofSteps; it++){

		std::cout << "time step " << it << std::endl;
		
		Vvt = l * sigmavt;		
		
		//Update the surface charge density histogram due to the currents:

		// Handling the currents in x:
		for (int j = 1; j<=ny; j++){
			for (int i = 1; i<=nx-1; i++){	
		
				Kt = ((-1)*(Vvt(i+1+nx*(j-1)) - Vvt(i+nx*(j-1)))*sigmas)/ax;
				sigmaht->SetCellContent(i, j, sigmaht->GetCellContent(i,j) - (Kt*Deltat*ay)/DeltaSn);			
				sigmaht->SetCellContent(i+1, j, sigmaht->GetCellContent(i+1,j) + (Kt*Deltat*ay)/DeltaSn);	
		
			}
		}
		
		// Handling the currents in y:
		for (int j = 1; j<=ny-1; j++){
			for (int i = 1; i<=nx; i++){	
		
				Kt = ((-1)*(Vvt(i+nx*j) - Vvt(i+nx*(j-1)))*sigmas)/ay;
				sigmaht->SetCellContent(i, j, sigmaht->GetCellContent(i,j) - (Kt*Deltat*ax)/DeltaSn);			
				sigmaht->SetCellContent(i, j+1, sigmaht->GetCellContent(i,j+1) + (Kt*Deltat*ax)/DeltaSn);	
		
			}
		}
		
		//Discharge currents:
		for (int j = 1; j<=ny; j++){
			for (int i = 1; i<=nx; i++){	
			 	if(connh->GetCellContent(i,j) < 1e20) {
			
					Rcon = (1/(2*sigmas)) + connh->GetCellContent(i,j);
					
					It = ((-1)*(Vvt(1+nx*(j-1)) - 0)) / Rcon;
					//cout << "Rcon = " << Rcon << endl;
					//cout << "It = " << It << endl;
					sigmaht->SetCellContent(i, j, sigmaht->GetCellContent(i,j) + (It*Deltat)/DeltaSn);		
					
				}
		
			}
		}
		
		//Update the sigmavt	
		for (int j = 1; j<=ny; j++){
			for (int i = 1; i<=nx; i++){
				sigmavt(i+nx*(j-1)) = sigmaht->GetCellContent(i,j);
			}
		}
		
		//sumt = sigmavt.Sum()*DeltaSn;
		//cout << "sum(" << it << ") = " << sumt << endl; // Sanity check (Charge Conservation).
	
	}
	
	char sigmahfName[16];
	sprintf(sigmahfName, "sigmah%i", tStepi+nofSteps); 
	TH2 *sigmahf; // Final surface charge density histogram
	
		
	sigmahf = new TH2D(Form("Sigmah%i(x,y)",tStepi+nofSteps), "Surface charge density", nx, -lx/2, lx/2, ny, -ly/2, ly/2) ;
	
	//Fill the final surface charge density histogram:	
	for (int j = 1; j<=ny; j++){ 
			for (int i = 1; i<=nx; i++){	
				sigmahf->SetCellContent(i, j, sigmaht->GetCellContent(i,j));
				//std::cout << sigmahtd->GetCellContent(i,j) << std::endl;
			}
		}

	
	sigmahf->Write(sigmahfName); // Write the histogram after transient calculation
	
	delete sigmahf;
	
	f.Close();
	
	delete sigmaht;
	delete connh;

}
