// this file adds a periodic charge distribution on the surface 
// sigma = sigma0 * sin(kx) e^(-t/tau)
// Chani will simulate the relaxation of the charge.

#include <iostream>
#include "Chani.h"
#include "TH2.h"
#include "TMath.h"

// compile and run with:
// g++ `root-config --glibs` -I `root-config --incdir` initialchargesin.C ; ./a.out

//void initialchargesin(){
int initialchargesin(){

Readp rp;
double lx = rp.Getlx();
double ly = rp.Getly();
double ax = rp.Getax();
double ay = rp.Getay();
int nx = rp.Getnx();
int ny = rp.Getny();
int Nm = rp.GetNm();

int N = nx*ny;
double DeltaSn = ax*ay;

TString fnamev = TString::Format("%sxy.root", dir);

// Reading of x & y vectors:	
TFile f(fnamev); 
		
TVectorD *xp; // Pointer for x-vector
f.GetObject("x-vector",xp);
TVectorD x(1,N);
x = *xp;
delete xp;

TVectorD *yp; // Pointer for y-vector
f.GetObject("y-vector",yp);
TVectorD y(1,N);
y = *yp;
delete yp;

f.Close();

double k = 1000;

cout << "k = " << k << endl;

double sigmaadd0 = (1e4*1.6e-19)/DeltaSn; //1e4 elementary charge

// add the initial periodic charge:

double sigmaadd;

TH2 *sigmah0;
sigmah0 = new TH2D("sigma0(x,y)", "Surface charge density at t = 0", nx, -lx/2, lx/2, ny, -ly/2, ly/2) ;
	
for (int j = 1; j <= ny; j++){
	for (int i =  1; i <= nx; i++){
	
	//std::cout << "i = " << i << std::endl;
	
	sigmaadd = sigmaadd0*cos(k*x(i+(j-1)*nx));
	sigmah0->SetCellContent(i,j,sigmaadd);
	
	}
}

TString fnameh = TString::Format("%sh.root", dir);

TFile f2(fnameh,"UPDATE"); // open the output file for histograms

sigmah0->Write("sigmah0"); // Write the surface with periodic initial charge
	
f2.Close();

}
